package com.shixing.sxedit;

import com.shixing.sxedit.delegate.SXTextAnimationDelegate;
import com.shixing.sxedit.types.SXTextAlignmentType;
import com.shixing.sxedit.types.SXTextDirectionType;
import com.shixing.sxedit.types.SXTextStrokeStyle;
import com.shixing.sxedit.util.TextUtils;

import java.util.ArrayList;
import java.util.List;
/**
 * 文字轨道
 * */

public class SXTextTrack extends SXRenderTrack implements SXTextAnimationDelegate {

    public SXTextTrack(long track, long manager, SXTrackOwnType type) {
        super(track, manager, type);
        mTrackType = SX_TRACK_TYPE_TEXT;
    }

    /**
     * 获取文字内容
     *
     * @return String
     * */
    public String getContent() {
        return nContent(mNativeManager, mOwnType.value, mTrackID);
    }
    /**
     * 设置文字内容
     * @param content 内容
     */
    public void setContent(String content) {
        if (content == null) {
            content = "";
        }

        if (mNativeManager > 0) {
            nSetContent(mNativeManager, mOwnType.value,mTrackID, content);
        }
    }

    /**
     * 获取描边宽度, 单位: px
     *
     * @return float
     * */
    public float getStrokeWidth() {
        return nStrokeWidth(mNativeManager, mOwnType.value, mTrackID);
    }

    /**
     * 设置描边宽度，默认为0
     *
     * @param strokeWidth 宽度
     */
    public void setStrokeWidth(float strokeWidth) {
        if (mNativeManager > 0 && strokeWidth >= 0) {
            nSetStrokeWidth(mNativeManager, mOwnType.value,  mTrackID, strokeWidth);
        }
    }

    public float getFontSize() {
        return nFontSize(mNativeManager, mOwnType.value, mTrackID);
    }
    /**
     * 设置文字大小，单位为像素
     *
     * @param fontSize 文字大小
     */
    public void setFontSize(float fontSize) {
        if (mNativeManager > 0 && fontSize > 0) {
            nSetFontSize(mNativeManager, mOwnType.value, mTrackID, fontSize);
        }
    }

    public int getTracking() {
        return nTracking(mNativeManager, mOwnType.value, mTrackID);
    }

    public void setTracking(int tracking) {
        if (mNativeManager > 0 && tracking >= 0) {
            nSetTracking(mNativeManager, mOwnType.value, mTrackID, tracking);
        }
    }

    public String getFontFile() {
        if (mNativeManager > 0) {
            return nFontFile(mNativeManager, mOwnType.value, mTrackID);
        }
        return null;
    }
    /**
     * 设置字体文件，默认使用初始设置的默认字体文件
     *
     * @param fontFile 字体文件本地路径
     */
    public void setFontFile(String fontFile) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(fontFile)) {
            nSetFontFile(mNativeManager, mOwnType.value, mTrackID, fontFile);
        }
    }

    public SXTextStrokeStyle getStrokeStyle() {
        int style = nStrokeStyle(mNativeManager, mOwnType.value, mTrackID);
        for (SXTextStrokeStyle value : SXTextStrokeStyle.values()) {
            if (value.ordinal() == style) {
                return value;
            }
        }
        return SXTextStrokeStyle.FillOverStroke;
    }
    /**
     * 设置描边绘制方式，默认描边画在文字之下
     *
     * @param style 描边方式
     */
    public void setStrokeStyle(SXTextStrokeStyle style) {
        if (mNativeManager > 0 && style != null) {
            nSetStrokeStyle(mNativeManager, mOwnType.value, mTrackID, style.ordinal());
        }
    }


    public SXTextDirectionType getDirection() {
        int dir = nDirection(mNativeManager, mOwnType.value, mTrackID);
        for (SXTextDirectionType value : SXTextDirectionType.values()) {
            if (value.ordinal() == dir) {
                return value;
            }
        }
        return SXTextDirectionType.Horizontal;
    }
    /**
     * 设置文字方向
     *
     * @param direction 横向或纵向
     */
    public void setDirection(SXTextDirectionType direction) {
        if (mNativeManager > 0 && direction != null) {
            nSetDirection(mNativeManager, mOwnType.value, mTrackID, direction.ordinal());
        }
    }

    public SXTextAlignmentType getAlignment() {
        if (mNativeManager > 0) {
            int alignment = nAlignment(mNativeManager, mOwnType.value, mTrackID);
            return SXTextAlignmentType.values()[alignment];
        }
        return SXTextAlignmentType.Left;
    }

    /**
     * 设置排版方式
     * @param type 排版方式
     */
    public void setAlignment(SXTextAlignmentType type) {
        if (mNativeManager > 0 && type != null) {
            nSetAlignment(mNativeManager, mOwnType.value, mTrackID, type.ordinal());
        }
    }
    /**
     * 设置填充颜色，默认为白色
     *
     * @param red 范围  [0, 1]
     * @param green 范围 [0, 1]
     * @param blue 范围 [0, 1]
     */
    public void setFillColor(float red, float green, float blue) {
        if (mNativeManager > 0) {
            nSetFillColor(mNativeManager, mOwnType.value, mTrackID, red, green, blue);
        }
    }
    /**
     * 设置填充颜色，默认为白色
     * @param color 填充颜色
     */
    public void setFillColor(int color) {
        if (mNativeManager > 0) {
            float r = ((color >> 16) & 0xff) / 255.0f;
            float g = ((color >>  8) & 0xff) / 255.0f;
            float b = ((color      ) & 0xff) / 255.0f;

            nSetFillColor(mNativeManager, mOwnType.value, mTrackID, r, g, b);
        }
    }

    /**
     * 设置填充颜色
     *
     * @param hexColorString, 十六进制颜色字符串, 比如 #00FF00, #00FF00FF
     * */
    public void setFillColor(String hexColorString) {
        if (mNativeManager > 0) {
            nSetFillColorByHexString(mNativeManager, mOwnType.value, mTrackID, hexColorString);
        }
    }

    /**
     * 设置描边颜色， 默认为白色
     *
     * @param red
     * @param green
     * @param blue
     */
    public void setStrokeColor(float red, float green, float blue) {
        if (mNativeManager > 0) {
            nSetStrokeColor(mNativeManager, mOwnType.value, mTrackID, red, green, blue);
        }
    }

    /**
     * 设置填充颜色，默认为白色
     * @param color 填充颜色
     */
    public void setStrokeColor(int color) {
        if (mNativeManager > 0) {
            float r = ((color >> 16) & 0xff) / 255.0f;
            float g = ((color >>  8) & 0xff) / 255.0f;
            float b = ((color      ) & 0xff) / 255.0f;

            nSetStrokeColor(mNativeManager, mOwnType.value, mTrackID, r, g, b);
        }
    }


    /**
     * 设置描边颜色
     *
     * @param hexColorString, 十六进制颜色字符串, 比如 #00FF00, #00FF00FF
     * */
    public void setStrokeColor(String hexColorString) {
        if (mNativeManager > 0) {
            nSetStrokeColorByHexString(mNativeManager, mOwnType.value, mTrackID, hexColorString);
        }
    }

    /**
     * 设置文字气泡
     *
     * @param resourcePath, 文字气泡本地路径
     * @return boolean
     * */
    public boolean setBubble(String resourcePath) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(resourcePath)) {
            nSetBubble(mNativeManager, mOwnType.value,  mTrackID, resourcePath);
        }
        return false;
    }

    /**
     * 删除文字气泡
     * */
    public void removeBubble() {
        if (mNativeManager > 0) {
            nRemoveBubble(mNativeManager, mOwnType.value, mTrackID);
        }
    }

    /**
     * 获取文字气泡素材对象
     *
     * @return SXResource
     * */
    public SXResource getBubbleResource() {
        if (mNativeManager > 0) {
            long source =  nBubbleResource(mNativeManager, mOwnType.value, mTrackID);
            if (source > 0) {
                return new SXResource(source);
            }
        }
        return null;
    }
    /**
     * 设置文字样式
     *
     * @param resourcePath, 文字样式本地路径
     * @return boolean
     * */
    public boolean setTextStyle(String resourcePath) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(resourcePath)) {
            return nSetTextStyle(mNativeManager, mOwnType.value, mTrackID, resourcePath);
        }
        return false;
    }

    /**
     * 重置文字样式
     *
     * */
    public void resetTextStyle() {
        if (mNativeManager > 0) {
            nResetTextStyle(mNativeManager, mOwnType.value, mTrackID);
        }
    }

    /**
     * 获取文字样式素材对象
     *
     * @return SXResource
     * */
    public SXResource getStyleResource() {
        if (mNativeManager > 0) {
            long source =  nStyleResource(mNativeManager, mOwnType.value, mTrackID);
            if (source > 0) {
                return new SXResource(source);
            }
        }
        return null;
    }

    @Override
    public SXTextAnimationEffect addTextAnimation(String textAnimationPath, int index) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(textAnimationPath)) {
            long effect = nAddTextAnimation(mNativeManager, mOwnType.value, mTrackID, textAnimationPath, index);
            if (effect > 0) {
                return new SXTextAnimationEffect(effect, mNativeManager, mTrackID);
            }
        }
        return null;
    }

    @Override
    public SXTextAnimationEffect addTextAnimationByCopy(SXTextAnimationEffect effect, int index) {
        if (mNativeManager > 0 && effect!= null) {
            long nEffect = nAddTextAnimationByCopy(mNativeManager, mOwnType.value, mTrackID, effect.getNativeEffect(), index);
            if (nEffect > 0) {
                return new SXTextAnimationEffect(nEffect, mNativeManager, mTrackID);
            }
        }
        return null;
    }

    @Override
    public boolean removeTextAnimation(String textAnimationId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(textAnimationId)) {
            return nRemoveTextAnimation(mNativeManager, mOwnType.value, mTrackID, textAnimationId);
        }
        return false;
    }

    @Override
    public SXTextAnimationEffect getTextAnimation(String textAnimationId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(textAnimationId)) {
            long effect = nGetTextAnimation(mNativeManager, mOwnType.value, mTrackID, textAnimationId);
            if (effect > 0) {
                return new SXTextAnimationEffect(effect, mNativeManager, mTrackID);
            }
        }
        return null;
    }

    @Override
    public int getNumTextAnimations() {
        if (mNativeManager > 0) {
            return nNumTextAnimations(mNativeManager, mOwnType.value, mTrackID);
        }
        return 0;
    }

    @Override
    public List<SXTextAnimationEffect> getTextAnimations() {
        long[] nAnims = nGetTextAnimations(mNativeManager, mOwnType.value, mTrackID);
        if (nAnims != null && nAnims.length > 0) {
            List<SXTextAnimationEffect> effects = new ArrayList<>(nAnims.length);
            for (long anim : nAnims) {
                effects.add(new SXTextAnimationEffect(anim, mNativeManager, mTrackID));
            }
            return effects;
        }
        return null;
    }
}
